/*
    Klat - A LaTeX editor for KDE
    
    Copyright (C) 2002-2004  Jori Liesenborgs (jori@lumumba.luc.ac.be)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "klatedittab.h"
#include "klatedittab.moc"
#include "klattooltab.h"
#include "klatoutputtab.h"
#include <kurl.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kprocess.h>
#include <Qt/qmime.h>
#include <Qt/qevent.h>
#include <stdio.h>
#include "klateditwidget.h"

KlatEditTab::KlatEditTab(QWidget *parent) : KTabWidget(parent)
{
	tooltab = 0;
	outputtab = 0;
	process = 0;
	counter = 0;
	connect(this,SIGNAL(currentChanged(int )),this,SLOT(slotPageChanged()));
	setAcceptDrops(true);
//	connect(this,SIGNAL(signalOutputRead(QString)),outputtab,SLOT(slotAddStdout(QString)));
//	connect(this,SIGNAL(signalErrorRead(QString)),outputtab,SLOT(slotAddStderr(QString)));

}

KlatEditTab::~KlatEditTab()
{
}

void KlatEditTab::slotFileNew()
{
	KlatEditWidget *w;
	char dummy[256];

	w = new KlatEditWidget(this);
	connect(w,SIGNAL(cursorPositionChanged()),this,SLOT(slotNewCursorPos()));
	
	sprintf(dummy,"Untitled %d",counter++);
	addTab(w,dummy);
	
	setCurrentIndex(count()-1);
}

void KlatEditTab::slotFileOpen()
{
	KUrl::List urls;
	KUrl::List::const_iterator it;
	
	KFileDialog dlg(KUrl(),"*.tex *.TEX *.bib *.BIB",this);
	
	dlg.setOperationMode(KFileDialog::Opening);
	dlg.setMode(KFile::Files);
	if (dlg.exec() == QDialog::Rejected)
		return;
	
	urls = dlg.selectedUrls();
	for (it = urls.begin() ; it != urls.end() ; ++it)
		slotOpenUrl(*it);

}

void KlatEditTab::slotOpenUrl(const KUrl &url)
{
	QString fname;
	KlatEditWidget *w;
	
	if (!url.isLocalFile())
	{
		KMessageBox::messageBox(this,KMessageBox::Information,
				"The file you specified isn't a local file","Can't open file");
		return;
	}
	
	w = new KlatEditWidget(this);
	
	if (!w->openUrl(url))
	{
		QString errstr = "Unable to open";
		
		errstr += url.prettyUrl();
		
		KMessageBox::messageBox(this,KMessageBox::Error,
				errstr,"Error!");
		delete w;
		return;
	}
	
	connect(w,SIGNAL(cursorPositionChanged()),this,SLOT(slotNewCursorPos()));
	
	if (getFilenameAndPath(url,&fname))
		addTab(w,fname);
	else
		addTab(w,url.prettyUrl());
	setCurrentIndex(count()-1);

	emit signalFileOpened(url);
}

void KlatEditTab::slotInsertString(const QString &str,int advance)
{
	KlatEditWidget *w;
	unsigned int row,col;
	
//	KMessageBox::error(this,str,"");
	w = (KlatEditWidget *)currentWidget();
	if (w == 0)
		return;
	row = w->cursorLine();
	col = w->cursorColumnReal();
	w->insertText(row,col,str);
	w->setCursorPositionReal(row,col+advance);
}

bool KlatEditTab::getFilenameAndPath(const KUrl &url,QString *fname,QString *path)
{
	QString fullpath = url.path();
	int i,length,fnamepos;
	bool done;
	
	if ((length = (int)fullpath.length()) == 0)
		return false;

	fnamepos = 0;
	for (i = length-1,done = false ; i >= 0 && !done ; i--)
	{
		if (fullpath[i] == '/')
		{
			done = true;
			fnamepos = i+1;
		}
	}

	if (!done)
		return false;
	
	if (fnamepos >= length)
		return false;

	if (fname)
		*fname = fullpath.right(length-fnamepos);
	if (path)
		*path = fullpath.left(fnamepos-1);
	
	return true;
}

void KlatEditTab::slotProcessExited()
{
	if (process->exitStatus() == QProcess::CrashExit)
		KMessageBox::messageBox(this,KMessageBox::Information,"A process exited with an error code. ","Error!");
	outputtab->finishedProcess();
	delete process;
	process=0;
}

void KlatEditTab::slotProcessError()
{
	KMessageBox::messageBox(this,KMessageBox::Information,"An error was detected in the process. ","Error!");
}

void KlatEditTab::slotReadStandardOutput()
{
    if(!process)
	return;
    QString stringOutput = process->readAllStandardOutput().data();
    if(!stringOutput.isEmpty())
	emit signalOutputRead(stringOutput);
}

void KlatEditTab::slotReadStandardError()
{
    if(!process)
	return;

    QString stringError = process->readAllStandardError().data();
    if(!stringError.isEmpty())
	emit signalErrorRead(stringError);
}

void KlatEditTab::executeCommand(const QString &cmd,const QString &cmdline,bool savefirst)
{
	KlatEditWidget *w;
	KUrl url;
	QString fname,path,commandline,basename;
	
	if(process)
	    QObject::disconnect(process,0,0,0);

	if (cmdline.contains('%') || savefirst) // check if we got a document
	{
		w = (KlatEditWidget *)currentWidget();
		if (!w)
		{
			KMessageBox::messageBox(this,KMessageBox::Information,"You are not editing a document","No document");
			return;
		}
		
		url = w->url();
		if (url.url().length() == 0)
		{
			KMessageBox::messageBox(this,KMessageBox::Information,"You must save the file first","No filename");
			return;
		}

		if (savefirst)
		{
			if (!w->save())
			{
				KMessageBox::messageBox(this,KMessageBox::Information,"Unable to save document before executing the command","Error!");
				return;
			}
		}

		if (!getFilenameAndPath(url,&fname,&path))
		{
			KMessageBox::messageBox(this,KMessageBox::Information,"Can't determine the filename or the path of the file","Error!");
			return;
		}

		basename = getBaseName(fname);
		commandline = cmdline;

		commandline.replace(QRegExp("%"),basename);
	}
	else
	{
		commandline = cmdline;
		path = "./";
	}
	process = new KProcess();
	
	disconnect(process,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(slotProcessExited()));
	disconnect(process,SIGNAL(error(QProcess::ProcessError)),this,SLOT(slotProcessError()));
	disconnect(process,SIGNAL(readyReadStandardOutput()),this,SLOT(slotReadStandardOutput()));
	disconnect(process,SIGNAL(readyReadStandardError()),this,SLOT(slotReadStandardError()));
	disconnect(this,SIGNAL(signalOutputRead(QString)),outputtab,SLOT(slotAddStdout(QString)));
	disconnect(this,SIGNAL(signalErrorRead(QString)),outputtab,SLOT(slotAddStderr(QString)));

	connect(process,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(slotProcessExited()));
	connect(process,SIGNAL(error(QProcess::ProcessError)),this,SLOT(slotProcessError()));
	connect(process,SIGNAL(readyReadStandardOutput()),this,SLOT(slotReadStandardOutput()));
	connect(process,SIGNAL(readyReadStandardError()),this,SLOT(slotReadStandardError()));
	connect(this,SIGNAL(signalOutputRead(QString)),outputtab,SLOT(slotAddStdout(QString)));
	connect(this,SIGNAL(signalErrorRead(QString)),outputtab,SLOT(slotAddStderr(QString)));
	
	process->setWorkingDirectory(path);
	(*process) << cmd << commandline;
	process->setNextOpenMode(QIODevice::ReadOnly);
	process->setOutputChannelMode(process->SeparateChannels);

	process->start();
	if(process->waitForStarted(1000))
	{
            outputtab->newProcess(cmd+" "+commandline);
	    process->closeWriteChannel();
	}
	else
	{
	    KMessageBox::messageBox(this,KMessageBox::Information,QString("Can't execute the command\"%s\". ").replace(QRegExp("%s"),cmd+" "+commandline),"Error!");
	    delete process;
	    process=0;
	}
}

QString KlatEditTab::getBaseName(const QString &fname)
{
	int i,len,pos;
	bool found;
	
	len = (int)fname.length();
	if (len == 0)
		return fname;
	pos = 0;	
	for (i = len-1,found = false ; !found && i >= 0 ; i--)
	{
		if (fname[i] == '.')
		{
			found = true;
			pos = i;
		}
	}
	if (!found)
		return fname;
	return fname.left(pos);
}

void KlatEditTab::slotFileClose()
{
	KlatEditWidget *w = (KlatEditWidget *)currentWidget();
	
	if (w == 0)
		return;
	if (w->isModified())
	{
		int status = KMessageBox::warningYesNo(this,"Document contains unsaved data. Close anyway?","File not saved");
	
		if (status != KMessageBox::Yes)
			return;
	}
	removePage(w);
	delete w;
	
	slotNewCursorPos();
}

void KlatEditTab::slotNewCursorPos()
{
	KlatEditWidget *w = (KlatEditWidget *)currentWidget();
	int line,col;
	
	if (!w)
	{
		line = -1;
		col = -1;
	}
	else
	{
		line = w->cursorLine();
		col = w->cursorColumn();
	}

	emit signalCursorPositionChanged(line,col);
}

void KlatEditTab::slotPageChanged()
{
	KlatEditWidget *w = (KlatEditWidget *)currentWidget();
	slotNewCursorPos();
	if(w)
	    emit signalNewDocument(w->getView());

}

bool KlatEditTab::areAllDocumentsSaved()
{
	int i,num;

	num = count();
	for (i = 0 ; i < num ; i++)
	{
		KlatEditWidget *w;
		
		w = (KlatEditWidget *)widget(i);
		if (w->isModified())
			return false;
	}
	return true;
}


void KlatEditTab::dropEvent(QDropEvent *e)
{
//	QStringList files;
	KUrl::List files;
//	QStringList::const_iterator it;
	KUrl::List::const_iterator it;
//	if (!QUriDrag::decodeLocalFiles(e,files))
//		return;
	if(!files.canDecode(e->mimeData()))
	    return;
	    
	files.fromMimeData(e->mimeData());
	
	for (it = files.begin() ; it != files.end() ; ++it)
	{
//		QString urlstr = "file:";
		
//		urlstr += (*it);//This is not actual more

//		KUrl url(urlstr);
		
//		if (url.isValid())
//		{
//			if (url.isLocalFile())
//				slotOpenUrl(url);
//		}
		if ((*it).isValid())
		{
			if ((*it).isLocalFile())
				slotOpenUrl(*it);
		}
	}
}

void KlatEditTab::dragEnterEvent(QDragEnterEvent *e)
{
//	e->accept(QUriDrag::canDecode(e));
	KUrl::List files;
	if(files.canDecode(e->mimeData()))
//	if(e->mimeData()->hasUrls())
	    e->acceptProposedAction();
}

void KlatEditTab::setFocus()
{
	KlatEditWidget *w = (KlatEditWidget *)currentWidget();

	if (!w)
		return;
	w->setFocus();
}

void KlatEditTab::slotNextDocument()
{
	int pos,num;

	if ((num = count()) <= 1)
		return;
	
	pos = currentIndex();
	pos++;
	if (pos >= num)
		pos = 0;
	setCurrentIndex(pos);
}

void KlatEditTab::slotPrevDocument()
{
	int pos,num;

	if ((num = count()) <= 1)
		return;
	
	pos = currentIndex();
	pos--;
	if (pos < 0)
		pos = num-1;
	setCurrentIndex(pos);
}
